//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "shared.fx"
#include "lighting.fx"
#include "fog.fx"

//------------------------------------------------------------------------------------------------------
//- Static parameters
//------------------------------------------------------------------------------------------------------

texture			Diffuse1;
texture			Diffuse2;
texture			LightMap;

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

struct	CVertexShaderInput
{
	float4	Position	:	POSITION;
	float3	Normal		:	NORMAL;
	float2	Diffuse1Uv	:	TEXCOORD0;	
	float2	Diffuse2Uv	:	TEXCOORD1;	
	float2	LightmapUv	:	TEXCOORD2;
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutput
{
	float4	Position			:	POSITION;
	float2	Diffuse1Uv			:	TEXCOORD0;	
	float2	Diffuse2Uv			:	TEXCOORD1;	
	float2	LightmapUv			:	TEXCOORD2;
	FOG_OPTION_VERTEX_FIELD
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutputLighting
{
	float4	Position			:	POSITION;
	float2	Diffuse1Uv			:	TEXCOORD0;	
	float2	Diffuse2Uv			:	TEXCOORD1;	
	float3	LightVector			:	TEXCOORD2;	
	FOG_OPTION_ON(float4	NormalVector : COLOR0;) // fog hack... store fog in NormalVector.a	
	FOG_OPTION_OFF(float3	NormalVector :	COLOR0;) 
	//float3	NormalVector		:	COLOR0;	
	float3	DiffuseColor		:	COLOR1;	
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutput	OpaLightmapGeomTech1Pass1			(const CVertexShaderInput input);
CVertexShaderOutputLighting	OpaLightmapGeomTech2Pass1	(const CVertexShaderInput input);

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutput	OpaLightmapGeomTech1Pass1 (const CVertexShaderInput input)
{
	CVertexShaderOutput output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);
		
	// Diffuse Uv output
	output.Diffuse1Uv = input.Diffuse1Uv;
	output.Diffuse2Uv = input.Diffuse2Uv;
		
	// LightmapUv Uv output
	output.LightmapUv = input.LightmapUv;
	
	// fog computation
	FOG_OPTION_COMPUTE(output, output.Position);
	
	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutputLighting	OpaLightmapGeomTech2Pass1 (const CVertexShaderInput input)
{
	CVertexShaderOutputLighting output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);
		
	// Diffuse Uv output
	output.Diffuse1Uv = input.Diffuse1Uv;
	output.Diffuse2Uv = input.Diffuse2Uv;
	
	// omnilight direction computation
	float4 omniDirection = ComputeVectorAndLength (ObjectLocalLightPosition[0],input.Position);
	
	// omnilight direction attenuation
	//omniDirection *= ComputeAttenuation (omniDirection.w,LightAttenuationFarStart[0],LightAttenuationFarEnd[0],LightAttenuationDelta[0]);
	
	output.DiffuseColor = DiffuseColor[0].xyz;
	output.DiffuseColor *= ComputeAttenuation (omniDirection.w,LightAttenuationFarStart[0],LightAttenuationFarEnd[0],LightAttenuationDelta[0]);
	
	// omnilight output to diffuse
	//output.LightVector.xyz = omniDirection * 0.5f + 0.5f;
	output.LightVector.xyz = omniDirection.xyz;
	output.NormalVector.xyz = input.Normal * 0.5f + 0.5f;	

	// fog computation
	FOG_OPTION_ON(output.NormalVector.a = fog_linear(output.Position));		
		
	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

sampler Diffuse1Sampler = sampler_state
{
	Texture		= <Diffuse1>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------

sampler Diffuse2Sampler = sampler_state
{
	Texture		= <Diffuse2>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------

sampler LightmapSampler = sampler_state
{
	Texture		= <LightMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------

sampler LightTextureSampler = sampler_state
{
	Texture		= <LightTexture>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
    AddressV	= CLAMP;
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


technique	tech1
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = ACCUMULATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		Sampler[0]	= <Diffuse1Sampler>;
		Sampler[1]	= <Diffuse2Sampler>;
		Sampler[2]	= <LightmapSampler>;
        
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= true;
		AlphaBlendEnable	= false;
		AlphaTestEnable		= false;
		FOG_OPTION_PARAMETERS;
		
		VertexShader = compile vs_1_1 OpaLightmapGeomTech1Pass1();
		
		PixelShader = 
		asm
		{
			ps_1_1
			
			tex			t0	// Diffuse1
			tex			t1	// Diffuse2
			tex			t2	// Light map
			
			mul			r0,t1,t1.a
			mad			r0,t0,1-t1.a,r0
			mul			r1,t2,t2.a
			mul_x4		r0,r0,r1
		};
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech2
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = ACCUMULATED_LIGHTING;
	string RenderingType = "Lighting";
>
{
	pass pass1
	{
		Sampler[0]	= <Diffuse1Sampler>;
		Sampler[1]	= <Diffuse2Sampler>;
		Sampler[2]	= <LightTextureSampler>;
	
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= false;
		AlphaBlendEnable	= true;
		SrcBlend			= ONE;
        DestBlend			= ONE;
		AlphaTestEnable		= false;
		FogEnable			= false;
		
		VertexShader = compile vs_1_1 OpaLightmapGeomTech2Pass1();
		
		PixelShader =
		#ifdef FOG_OPTION_ENABLE 
		asm
		{
			ps_1_1
			
			tex			t0	// Diffuse1
			tex			t1	// Diffuse2
			tex			t2	// Renormalized light vector
			
			mul			r0,t1,t1.a
			mad			r0,t0,1-t1.a,r0
			dp3_sat		r1,t2_bx2,v0_bx2
			mul			r1.rgb,r1,v1
			mul_x2		r0,r0,r1
			
			// fog hack
			mul_sat r0.rgb, r0, v0.a
		};
		#else 
		asm
		{
			ps_1_1
			
			tex			t0	// Diffuse1
			tex			t1	// Diffuse2
			tex			t2	// Renormalized light vector
			
			mul			r0,t1,t1.a
			mad			r0,t0,1-t1.a,r0
			dp3_sat		r1,t2_bx2,v0_bx2
			mul			r1.rgb,r1,v1
			mul_x2		r0,r0,r1
		};
		#endif
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	techTnL_0
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = TNL_ONLY;
	int LightingType = ACCUMULATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		WorldTransform[0]	= <WorldCameraProjection>;
		IndexedVertexBlendEnable = false;
		
		Texture[0]			= <LightMap>;
		Texture[1]			= <Diffuse1>;
		TexCoordIndex[0]	= 2;
		TexCoordIndex[1]	= 0;
		TextureFactor		= {0.0f,0.0f,0.0f,0.0f};
		
		ColorArg1[0]	= TEXTURE;
		ColorArg2[0]	= TFACTOR;
		ColorOp[0]		= BLENDTEXTUREALPHA;
		AlphaArg1[0]	= TEXTURE;
		AlphaOp[0]		= SELECTARG1;
		
		ColorArg1[1]	= TEXTURE;
		ColorArg2[1]	= CURRENT;
		AlphaArg1[1]	= TEXTURE;
		AlphaOp[1]		= SELECTARG1;
		ColorOp[1]		= MODULATE4X;
		
		ColorOp[2]		= DISABLE;
		AlphaOp[2]		= DISABLE;
		
		MinFilter[0]		= LINEAR;
		MagFilter[0]		= LINEAR;
		MipFilter[0]		= LINEAR;
		
		MinFilter[1]		= LINEAR;
		MagFilter[1]		= LINEAR;
		MipFilter[1]		= LINEAR;
		
		AddressU[0]			= WRAP;
        AddressV[0]			= WRAP;
        
        AddressU[1]			= WRAP;
        AddressV[1]			= WRAP;
				
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= true;
		AlphaBlendEnable	= false;
		AlphaTestEnable		= true;
		AlphaRef			= 192;
		AlphaFunc			= GREATEREQUAL;
		
		VertexShader 		= NULL;		
		PixelShader 		= NULL;
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "mesh_shadow.fx"
#include "mesh_shadow_projector.fx"

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
